/*

    File: CModel.h

    Copyright (C) 2000-2007 Christopher Moore (christopher.e.moore@gmail.com)
	  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#ifndef CMODEL_H
#define CMODEL_H

#include "../m_math3d/vector.h"
#include "../m_math3d/quat.h"
#include "../m_math3d/geometry.h"

typedef struct edge_s {
	int		t[2];	//two triangle IDs
	int		v[2];	//two vtx IDs
//	Vector	normal;	//normalized average of two triangle normals
} edge_t;

class CModel {
public:
	CModel() {Reset();}
	~CModel() {Unload();}

	bool Load(const char *filename);
	void Unload(void);
	void Reset(void);

	void Transform(const quat_t *r, const vector_t *s, const vector_t *x);
	bool Copy(CModel *src);

	char	title[64];

	int			vtxCount;
	vector_t	*vtx;		//vtxCount

	int			planeCount;
	plane_t		*plane;		//triCount

	int			edgeCount;
	edge_t		*edge;		//edgeCount

	int			triCount;
	int			*tri;		//3*triCount

	box_t		bbox;
	scalar_t	radius;	//stealing Quake1 ideas... maximum magnitude vertex distance - ONLY USED FOR DEBUG NORMAL DISPLAY!!!
	
	scalar_t	volume;

	void CalcVolume(void);	//calc the volume var - done upon construction/modification
	void CalcInertia(const vector_t *scale, vector_t *vres);	//calc the inertia vector based on the 'scale' vector
	void CalcPlanes(void);	//calc the plane equations
	void CalcBounds(void);	//calc the bbox / radius

	//sets 'depth' to the greatest distance from any plane in the cmodel to the vector specified
	//returns the ID of the plane closest to 'vec'
	int PointSurfaceDepth(const vector_t *vec, scalar_t *depth);

	scalar_t Traceline(const line_t &line, vector_t *res) const;
};

#endif

//second system involves triangle array / triangle normal array / no edge array